UNIT UCA_BINI_Descriptor;
{$mode objfpc}{$H+}
INTERFACE
USES
    Classes
  , Sysutils
  ;



TYPE
  TR_BINI_Header = Record
    ID         : ARRAY[ 1 .. 4 ] OF CHAR;
    Version    : INT32;
    FileOffset : INT32;
  END ;


TYPE
  TR_BINI_Section_Pos = Record
    Name   : Int16;
    Entrys : Int16;
  END ;



TYPE
  TR_BINI_KeyWord_Pos = Record
    Key    : Int16;
    Values : Int8;
  END ;



TYPE
  TR_BINI_Value_Pos = Record
    Value_Typ : Byte;
    CASE LongInt OF
      1 : ( Value_LI     : int32 );
      2 : ( Value_Double : Double  );
      3 : ( Value_Int    : Int16 );
  END;


FUNCTION Decrypt_BINI( iv_FileName : String ) : Boolean;

VAR
  GO_FileStream : TFileStream;
  GO_Header     : TR_BINI_Header;





IMPLEMENTATION



{===================================================================================================
*
*
*
===================================================================================================}


FUNCTION Read_BINI_String( iv_Offset : Int64 ) : String;
VAR lv_c : Char;
BEGIN
  Result := '';
  go_FileStream.Seek( iv_Offset, soFromBeginning );
  lv_c := ' ';
  WHILE lv_c <> #0 DO BEGIN
    go_FileStream.Read( lv_c, SizeOF( lv_c ) );
    IF lv_c = #0 THEN Continue;
    Result := Result + lv_c;
  END;
END;


FUNCTION BINI_Read_longInt( VAR cv_StartPos : Int64; iv_Offset : INT64 ) : String;
VAR lv_Distanz : INT32; lv_Offset : INT64; lv_s : String;
BEGIN
  go_FileStream.Read( lv_Distanz, SizeOF( lv_Distanz ) );
  cv_StartPos := go_FileStream.Position;
  lv_Offset   := iv_Offset + lv_Distanz;
  lv_s        := Read_BINI_String( lv_Offset );
  go_FileStream.Seek( cv_StartPos, soFromBeginning );
  Result      := lv_s;
END;



FUNCTION BINI_Read_String( VAR cv_StartPos : Int64; iv_Offset : INT64 ) : String;
VAR lv_Distanz : INT16; lv_Offset : INT64; lv_s : String;
BEGIN
  go_FileStream.Read( lv_Distanz, SizeOF( lv_Distanz ) );
  cv_StartPos := go_FileStream.Position;
  lv_Offset   := iv_Offset + lv_Distanz;
  lv_s        := Read_BINI_String( lv_Offset );
  go_FileStream.Seek( cv_StartPos, soFromBeginning );
  Result      := lv_s;
END;



{===================================================================================================
1 Byte für den Datentyp
===================================================================================================}
FUNCTION READ_Values( VAR cv_StartPos : Int64; iv_Value_Cnt : Int8 ) : String;
VAR lv_Value_Type : Byte; lv_Pos : int64; lv_s : String; lv_i : Integer; lv_Offset : int64; lv_Dist_32 : int32; lv_Dist_16 : Int16;
BEGIN
  Result := '';
  lv_Offset := GO_Header.FileOffset;
  FOR lv_i := 1 TO iv_Value_Cnt DO BEGIN
    lv_Value_Type := go_FileStream.Read( lv_Value_Type, SizeOF( lv_Value_Type ) );
    cv_StartPos   := go_FileStream.Position;
    CASE lv_Value_Type OF
      1 : BEGIN
            lv_s := BINI_Read_longInt( cv_StartPos, lv_Offset );
          END;
      2 : BEGIN
          END;
      3 : BEGIN
            lv_s := BINI_Read_String( cv_StartPos, lv_Offset );
          END;
    END;
    Result := Result + lv_s;
  END ;
END;


{===================================================================================================
2 Byte's für die Position des Schlüsselwortes
1 Byte für die Anzahl der Werte
===================================================================================================}
FUNCTION READ_KeyWords( VAR cv_StartPos : Int64; iv_Keyword_CNT : Int16 ) : Boolean;
VAR lv_i : Integer; lv_Values, lv_s : String; lv_Offset : int64; lv_Keyword_Pos : int16; lv_ValueCNT : INT8;
BEGIN
  FOR lv_i := 1 TO iv_Keyword_CNT do BEGIN
    go_FileStream.Read( lv_Keyword_Pos, SizeOF( lv_Keyword_Pos ) );
    go_FileStream.Read( lv_ValueCNT, SizeOF( lv_ValueCNT ) );
    cv_StartPos        := go_FileStream.Position;

    lv_Offset          := GO_Header.FileOffset + lv_Keyword_Pos;
    lv_s               := Read_BINI_String( lv_Offset );
    go_FileStream.Seek( cv_StartPos, soFromBeginning );

    lv_Values  := READ_Values( cv_StartPos, lv_ValueCNT );
    lv_s := '';
  END;
END;


{===================================================================================================
2 Byte's für die Position des Sektions Namens
2 Byte's für die Anzahl der Schlüsselworte
===================================================================================================}
FUNCTION READ_Sections( VAR cv_StartPos, cv_Txt_Offset : Int64 ) : Boolean;
VAR lv_SEC_Offset : Int16; lv_KeyWord_CNT : Int16; lv_OffSet : Int64; lv_s : String;
BEGIN
  cv_StartPos := go_FileStream.Position;
  go_FileStream.Read( lv_SEC_Offset, SizeOF( lv_SEC_Offset ) );
  go_FileStream.Read( lv_KeyWord_CNT, SizeOF( lv_KeyWord_CNT ) );
  cv_StartPos := go_FileStream.Position;

  lv_OffSet   := cv_Txt_Offset + lv_SEC_Offset;
  lv_s        := Read_BINI_String( lv_OffSet );
  go_FileStream.Seek( cv_StartPos, soFromBeginning );
  cv_StartPos := go_FileStream.Position;

  READ_KeyWords( cv_StartPos, lv_KeyWord_CNT );
END;




FUNCTION Decrypt_BINI( iv_FileName : String ) : Boolean;
VAR lv_Txt_Offset : INT64; lv_StartPos : INT64;
BEGIN
  go_FileStream := TFileStream.Create( iv_FileName, fmOpenRead );
  go_FileStream.Read( GO_Header, SizeOF( GO_Header ) );
  IF GO_Header.ID <> 'BINI' THEN BEGIN
    EXIT( False );
  END;
  lv_Txt_Offset := GO_Header.FileOffset;
  lv_StartPos   := go_FileStream.Position;
  WHILE go_FileStream.Position < go_FileStream.Size DO BEGIN
    READ_Sections( lv_StartPos, lv_Txt_Offset );
  END;
  FreeAndNil( go_FileStream );
END;



END .

